@@ -673,3 +673,71 @@ test_that("model.frame", {
     m4 <- lmer(Reaction ~ log(1+Days) + (1 | Subject), sleepstudy)
     ee(m4, c("Reaction","log(1 + Days)"))
 })
+
+
+context("influence measures")
+
+d <- as.data.frame(ChickWeight)
+colnames(d) <- c("y", "x", "subj", "tx")
+dNAs <- d
+dNAs$y[c(1, 3, 5)] <- NA
+fitNAs <- lmer(y ~ tx*x + (x | subj), data = dNAs,
+               na.action=na.exclude)
+
+test_that("influence/hatvalues works", {
+    ifm1 <- influence(fm1)
+    expect_equal(unname(head(ifm1$hat)),
+                 c(0.107483311203734, 0.102096105816528,
+                   0.0980557017761242, 0.0953620990825215, 
+                   0.0940152977357202, 0.0940152977357202),
+                 tolerance=1e-6)
+    expect_equal(nrow(dNAs),length(hatvalues(fitNAs)))
+})
+
+test_that("rstudent", {
+    rfm1 <- rstudent(fm1)
+    expect_equal(unname(head(rfm1)),
+                 c(-1.45598270922089, -1.49664543508657, -2.11747425025103,
+                   -0.0729690066951975, 0.772716397142335, 2.37859408861768),
+                 tolerance=1e-6)
+    expect_equal(nrow(dNAs),length(rstudent(fitNAs)))
+})
+
+test_that("cooks distance", {
+    expect_equal(
+        unname(head(cooks.distance(fm1))),
+        c(0.127645976734753, 0.127346548123793, 0.243724627125036, 0.000280638917214881, 
+          0.0309804642689636, 0.293554225380831),
+        tolerance=1e-6)
+        expect_equal(nrow(dNAs),length(cooks.distance(fitNAs)))
+})
+
+## tweaked example so estimated var = 0
+zerodat <- data.frame(x=seq(0,1,length.out=120),
+                      f=rep(1:3,each=40))
+zerodat$y1 <- simulate(~x+(1|f),
+                      family=gaussian,
+                      seed=102,
+                      newparams=list(beta=c(1,1),
+                                     theta=c(0.001),
+                                     sigma=1),
+                      newdata=zerodat)[[1]]
+zerodat$y2 <- simulate(~x+(1|f),
+                      family=poisson,
+                      seed=102,
+                      newparams=list(beta=c(1,1),
+                                     theta=c(0.001)),
+                      newdata=zerodat)[[1]]
+
+test_that("rstudent matches for zero-var cases",
+{
+    lmer_zero <- lmer(y1~x+(1|f), data=zerodat)
+    glmer_zero <- glmer(y2~x+(1|f),family=poisson, data=zerodat)
+    lm_zero <- lm(y1~x, data=zerodat)
+    glm_zero <- glm(y2~x,family=poisson, data=zerodat)
+    expect_equal(suppressWarnings(rstudent(glmer_zero)),
+                 rstudent(glm_zero),
+                 tolerance=0.01)
+    expect_equal(suppressWarnings(rstudent(lmer_zero)),
+                 rstudent(lm_zero),tolerance=0.01)
+})
